import { __decorate, __param, __metadata, __awaiter } from 'tslib';
import { InjectionToken, Optional, Inject, ElementRef, Input, Directive, TemplateRef, ViewContainerRef, NgZone, ContentChildren, QueryList, HostListener, EventEmitter, Renderer2, Attribute, Output, ChangeDetectorRef, LOCALE_ID, HostBinding, Injector, Self, Pipe, ɵɵdefineInjectable, Injectable, NgModule } from '@angular/core';
import { DragController, DropController, LocalStorage, SessionStorage } from '@anjuna/core';
import { NgControl, NgForm, FormGroupDirective, NG_VALUE_ACCESSOR, NG_VALIDATORS, NgModel, FormControl } from '@angular/forms';
import { Subject } from 'rxjs';

const LANGUAGE_CODE = new InjectionToken('LanguageCode');

let BaseTranslationPipe = class BaseTranslationPipe {
    constructor(language) {
        this.language = language;
    }
    transform(value) {
        if (!this.translations[this.language]) {
            console.warn(`Translation file for '${this.language}' does not exist.`);
            return value;
        }
        return this.translations[this.language][value] || value;
    }
};
BaseTranslationPipe = __decorate([
    __param(0, Optional()), __param(0, Inject(LANGUAGE_CODE)),
    __metadata("design:paramtypes", [Object])
], BaseTranslationPipe);

/**
 * @description A wrapper around the HTML 5 Drag API.
 */
let DraggableDirective = class DraggableDirective {
    constructor(element) {
        this.element = element;
        this.dataFn = () => this.getData();
        this.controller = new DragController(this.element.nativeElement, this.dataFn).connect();
    }
    getData() {
        return this.anjDraggable;
    }
};
DraggableDirective.ctorParameters = () => [
    { type: ElementRef }
];
__decorate([
    Input(),
    __metadata("design:type", Object)
], DraggableDirective.prototype, "anjDraggable", void 0);
DraggableDirective = __decorate([
    Directive({
        selector: '[anjDraggable]'
    }),
    __metadata("design:paramtypes", [ElementRef])
], DraggableDirective);

/**
 * @description A wrapper around the HTML 5 Drop API.
 */
let DroppableDirective = class DroppableDirective {
    constructor(element) {
        this.element = element;
        this.controller = new DropController(this.element.nativeElement).connect();
    }
};
DroppableDirective.ctorParameters = () => [
    { type: ElementRef }
];
DroppableDirective = __decorate([
    Directive({
        selector: '[anjDroppable]'
    }),
    __metadata("design:paramtypes", [ElementRef])
], DroppableDirective);

class DomOutlet {
    constructor(targetElement) {
        this.targetElement = targetElement;
    }
    attachTemplate(template) {
        if (this.template.zone.isStable) {
            this._viewRef = this.template.zone.run(() => {
                return template.viewContainerRef.createEmbeddedView(template.templateRef, template.context);
            });
        }
        else {
            this._viewRef = template.viewContainerRef.createEmbeddedView(template.templateRef, template.context);
        }
        this._viewRef.rootNodes.forEach(rootNode => this.targetElement.appendChild(rootNode));
        this._viewRef.detectChanges();
        this.setDisposeFn((() => {
            const index = template.viewContainerRef.indexOf(this._viewRef);
            if (index !== -1) {
                template.viewContainerRef.remove(index);
            }
        }));
        return this._viewRef;
    }
    updateContext(context) {
        for (const prop in context) {
            if (context.hasOwnProperty(prop)) {
                this._viewRef.context[prop] = context[prop];
            }
        }
        this._viewRef.detectChanges();
    }
    setDisposeFn(fn) {
        this.disposeFn = fn;
    }
    attach(template) {
        this.template = template;
        return this.attachTemplate(template);
    }
    detach() {
        if (this.template) {
            this.template.setAttachedHost(null);
            this.template = null;
        }
        this.invokeDisposeFn();
    }
    dispose(removeParent = true) {
        this.disposeFn();
        if (removeParent && this.targetElement.parentNode != null) {
            this.targetElement.parentNode.removeChild(this.targetElement);
        }
        return undefined;
    }
    invokeDisposeFn() {
        if (this.disposeFn) {
            this.disposeFn();
            this.disposeFn = null;
        }
    }
}

class Template {
    constructor(templateDirective, context) {
        this.templateDirective = templateDirective;
        this.context = context;
    }
    get origin() {
        return this.templateDirective.template.elementRef;
    }
    get templateRef() {
        return this.templateDirective.template;
    }
    get viewContainerRef() {
        return this.templateDirective.viewContainerRef;
    }
    get zone() {
        return this.templateDirective.zone;
    }
    attach(host, context = this.context) {
        this.context = context;
        this.host = host;
        return host.attach(this);
    }
    detach() {
        this.context = undefined;
        if (this.host) {
            const host = this.host;
            this.host = null;
            host.detach();
        }
    }
    setAttachedHost(host) {
        this.host = host;
    }
}

let TemplateDirective = class TemplateDirective {
    constructor(_template, _viewRef, _zone) {
        this._template = _template;
        this._viewRef = _viewRef;
        this._zone = _zone;
    }
    set anjTemplateOf(name) {
        this._name = name;
    }
    get name() {
        return this._name;
    }
    get template() {
        return this._template;
    }
    get viewContainerRef() {
        return this._viewRef;
    }
    get zone() {
        return this._zone;
    }
};
TemplateDirective.ctorParameters = () => [
    { type: TemplateRef },
    { type: ViewContainerRef },
    { type: NgZone }
];
__decorate([
    Input(),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], TemplateDirective.prototype, "anjTemplateOf", null);
TemplateDirective = __decorate([
    Directive({
        selector: '[anjTemplate]'
    }),
    __metadata("design:paramtypes", [TemplateRef, ViewContainerRef, NgZone])
], TemplateDirective);

var AnjTable_1;
/* tslint:disable:directive-selector */
let AnjTable = AnjTable_1 = class AnjTable {
    constructor(elRef) {
        this.outletRows = [];
        this.element = elRef.nativeElement;
        this.element.renderer = this;
    }
    static rowContext(row, index, column) {
        return {
            $implicit: row,
            column,
            index
        };
    }
    static updateContext(outlet, context) {
        outlet.updateContext(context);
    }
    ngOnDestroy() {
        this.outletRows.map((rowMap) => {
            for (const templateName in rowMap) {
                if (rowMap[templateName]) {
                    rowMap[templateName] = rowMap[templateName].dispose();
                }
            }
            return rowMap;
        });
    }
    /**
     * Fixes https://git.zias.io/ui-foundation/anjuna/issues/876
     * When the columns change, destory all portals without killing the parent node
     */
    onColumnsUpdated() {
        this.outletRows.map((rowMap) => {
            for (const templateName in rowMap) {
                if (rowMap[templateName]) {
                    rowMap[templateName] = rowMap[templateName].dispose(false);
                }
            }
            return rowMap;
        });
    }
    /* These next three methods are a mess of duplicated logic - they will probably stay that way until feature complete, then I'll clean em up */
    cellTemplate(cell, offset, col, row, rowIndex, template) {
        const rowMap = this.outletRows[rowIndex] || {};
        const outlet = rowMap[col.name];
        if (!cell) {
            /**
             * Removing the line:
             * rowMap[col.name] = outlet ? outlet.dispose() : undefined;
             * which fixes https://git.zias.io/ui-foundation/anjuna/issues/850.
             * Like the detail situation, I do not understand why this works..
             */
            return;
        }
        if (outlet) {
            if (cell !== outlet.targetElement) {
                this.updateOutlet(outlet, template, cell, AnjTable_1.rowContext(row, rowIndex, col));
            }
            else {
                AnjTable_1.updateContext(outlet, AnjTable_1.rowContext(row, rowIndex, col));
            }
        }
        else {
            rowMap[col.name] = this.createOutlet(cell, template, AnjTable_1.rowContext(row, rowIndex, col));
            this.outletRows[rowIndex] = rowMap;
        }
        col.initCell(cell, offset);
    }
    rowTemplate(templateName, targetEl, row, rowIndex) {
        // Dispose row template no longer used for that index
        this.outletRows[rowIndex] = this.filterRowTemplates(this.outletRows[rowIndex], templateName);
        const outlet = this.outletRows[rowIndex][templateName];
        if (!targetEl) {
            this.outletRows[rowIndex][templateName] = outlet ? outlet.dispose() : undefined;
            return;
        }
        if (outlet) {
            if (targetEl !== outlet.targetElement) {
                this.updateOutlet(outlet, templateName, targetEl, AnjTable_1.rowContext(row, rowIndex));
            }
            else {
                AnjTable_1.updateContext(outlet, AnjTable_1.rowContext(row, rowIndex));
            }
        }
        else {
            this.outletRows[rowIndex][templateName] = this.createOutlet(targetEl, templateName, AnjTable_1.rowContext(row, rowIndex));
            this.outletRows[rowIndex] = this.outletRows[rowIndex];
        }
    }
    fromTemplate(templateName, targetEl, row, rowIndex) {
        const rowMap = this.outletRows[rowIndex] || {};
        const outlet = rowMap[templateName];
        if (!targetEl) {
            /**
             * Removing the line:
             * rowMap[templateName] = outlet ? outlet.dispose() : undefined;
             * is part of the fix for https://git.zias.io/ui-foundation/anjuna/issues/750
             * However, I don't grok at all how/why this works. I don't buy it.
             * So when we eventually get an issue related to master detail, start here.
             * Put that line back, and fix it right.
             */
            return;
        }
        if (outlet) {
            if (targetEl !== outlet.targetElement) {
                this.updateOutlet(outlet, templateName, targetEl, AnjTable_1.rowContext(row, rowIndex));
            }
            else {
                AnjTable_1.updateContext(outlet, AnjTable_1.rowContext(row, rowIndex));
            }
        }
        else {
            rowMap[templateName] = this.createOutlet(targetEl, templateName, AnjTable_1.rowContext(row, rowIndex));
            this.outletRows[rowIndex] = rowMap;
        }
    }
    createOutlet(target, templateName, context) {
        const outlet = new DomOutlet(target);
        outlet.attach(this.createTemplate(templateName, context));
        return outlet;
    }
    createTemplate(templateName, context) {
        const templateRef = this.templates.find(t => t.name === templateName);
        return new Template(templateRef, context);
    }
    updateOutlet(outlet, templateName, targetEl, context) {
        outlet.targetElement = targetEl;
        outlet.detach();
        outlet.attach(this.createTemplate(templateName, context));
    }
    /**
     * Disposes of row templates no longer used in a given row. This ONLY happens if they have specified more than one
     * Template and the end user sorts or filters or otherwise changes the order. If only one template is specified,
     * the output will always match the input -r
     */
    filterRowTemplates(rowMap = {}, templateName) {
        const updated = {};
        Object.keys(rowMap).forEach(k => {
            if (k === templateName) {
                updated[k] = rowMap[k];
            }
            else {
                rowMap[k].dispose(false);
            }
        });
        return updated;
    }
};
AnjTable.ctorParameters = () => [
    { type: ElementRef }
];
__decorate([
    ContentChildren(TemplateDirective),
    __metadata("design:type", QueryList)
], AnjTable.prototype, "templates", void 0);
__decorate([
    HostListener('anjColumnsDidUpdate'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], AnjTable.prototype, "onColumnsUpdated", null);
AnjTable = AnjTable_1 = __decorate([
    Directive({
        selector: 'anj-table:not([anjIgnore])',
    }),
    __metadata("design:paramtypes", [ElementRef])
], AnjTable);

var TransitionState;
(function (TransitionState) {
    TransitionState[TransitionState["Enter"] = 0] = "Enter";
    TransitionState[TransitionState["Leave"] = 1] = "Leave";
    TransitionState[TransitionState["EnterTo"] = 2] = "EnterTo";
    TransitionState[TransitionState["Inactive"] = 3] = "Inactive";
})(TransitionState || (TransitionState = {}));

var TransitionDirective_1;
let TransitionDirective = TransitionDirective_1 = class TransitionDirective {
    constructor(renderer, element, mode, name) {
        this.renderer = renderer;
        this.element = element;
        this.mode = mode;
        this.name = name;
        this.state = TransitionState.EnterTo;
        this.end = new EventEmitter();
        this.mode = this.mode ? this.mode : 'in-out';
        this.name = this.name ? this.name : 'transition';
    }
    ngOnChanges(changes) {
        this.endTransition();
        if (!changes.anjTransition.previousValue && !changes.anjTransition.currentValue && this.mode !== 'out') {
            this.initEnter();
        }
        else if (!changes.anjTransition.previousValue && changes.anjTransition.currentValue) {
            this.transitionIn();
        }
        else {
            this.transitionOut();
        }
    }
    handleTransition(event) {
        if (event.target === this.element.nativeElement && TransitionDirective_1.supportedTransitions.indexOf(event.propertyName) > -1) {
            event.stopPropagation();
            this.end.emit(this.state);
            if (this.state === TransitionState.Enter) {
                this.endTransition();
            }
            else {
                if (this.mode !== 'out' && this.anjTransition) {
                    this.transitionIn();
                }
                else {
                    this.endTransition();
                }
            }
        }
    }
    initEnter() {
        this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-enter-to`);
    }
    transitionIn() {
        if (this.mode !== 'out') {
            this.state = TransitionState.Enter;
            this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-enter-to`);
            this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-leave`);
            this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-leave-active`);
            this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-enter-active`);
            this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-enter`);
        }
        else {
            this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-enter-to`);
            this.endTransition();
        }
    }
    transitionOut() {
        this.state = TransitionState.Leave;
        if (this.mode !== 'in') {
            this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-leave-active`);
            this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-leave`);
        }
        else {
            this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-enter-to`);
            this.state = TransitionState.EnterTo;
            if (this.anjTransition) {
                window.setTimeout(() => {
                    this.transitionIn();
                });
            }
        }
    }
    endTransition() {
        this.state = TransitionState.Inactive;
        this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-enter-active`);
        this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-leave-active`);
        this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-enter`);
        this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-leave`);
        if (!this.anjTransition) {
            this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-enter-to`);
        }
    }
};
TransitionDirective.supportedTransitions = ['opacity', 'transform'];
TransitionDirective.ctorParameters = () => [
    { type: Renderer2 },
    { type: ElementRef },
    { type: undefined, decorators: [{ type: Attribute, args: ['mode',] }] },
    { type: String, decorators: [{ type: Attribute, args: ['name',] }] }
];
__decorate([
    Input(),
    __metadata("design:type", Object)
], TransitionDirective.prototype, "anjTransition", void 0);
__decorate([
    Output(),
    __metadata("design:type", EventEmitter)
], TransitionDirective.prototype, "end", void 0);
__decorate([
    HostListener('transitiend', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [TransitionEvent]),
    __metadata("design:returntype", void 0)
], TransitionDirective.prototype, "handleTransition", null);
TransitionDirective = TransitionDirective_1 = __decorate([
    Directive({
        selector: '[anjTransition]'
    }),
    __param(2, Attribute('mode')),
    __param(3, Attribute('name')),
    __metadata("design:paramtypes", [Renderer2,
        ElementRef, String, String])
], TransitionDirective);

const TIME_TIMEZONE_STRING = new InjectionToken('TimeTimezoneString');
const TIME_MERIDIEM = new InjectionToken('TimeMeridiem');
const DATE_TIMEZONE_OVERRIDE = new InjectionToken('DateTimezoneOverride');

/* tslint:disable:directive-selector */
let LocaleDirective = class LocaleDirective {
    constructor(changeRef, _locale, _lang, _timezone) {
        this.changeRef = changeRef;
        if (_locale) {
            this._locale = _locale;
            this._lang = _lang ? _lang : this._locale ? this._locale.split('-')[0] : 'en';
        }
        if (_timezone) {
            this._timezone = _timezone;
        }
        this.changeRef.markForCheck();
    }
};
LocaleDirective.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LOCALE_ID,] }] },
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LANGUAGE_CODE,] }] },
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [DATE_TIMEZONE_OVERRIDE,] }] }
];
__decorate([
    HostBinding('attr.locale'),
    __metadata("design:type", String)
], LocaleDirective.prototype, "_locale", void 0);
__decorate([
    HostBinding('attr.lang'),
    __metadata("design:type", String)
], LocaleDirective.prototype, "_lang", void 0);
__decorate([
    HostBinding('attr.timezone'),
    __metadata("design:type", String)
], LocaleDirective.prototype, "_timezone", void 0);
LocaleDirective = __decorate([
    Directive({
        selector: 'anj-locale:not([manual])',
    }),
    __param(1, Optional()), __param(1, Inject(LOCALE_ID)),
    __param(2, Optional()), __param(2, Inject(LANGUAGE_CODE)),
    __param(3, Optional()), __param(3, Inject(DATE_TIMEZONE_OVERRIDE)),
    __metadata("design:paramtypes", [ChangeDetectorRef, String, String, String])
], LocaleDirective);

let ValueAccessor = class ValueAccessor {
    constructor(element, parentForm, parentGroup, injector, changeRef) {
        this.element = element;
        this.injector = injector;
        this.changeRef = changeRef;
        /**
         * Enable two-way binding
         */
        this.valueChange = new EventEmitter();
        this.onChange = () => { };
        this.onTouched = () => { };
        this.parent = parentForm ? parentForm : parentGroup ? parentGroup : null;
    }
    get isInvalid() {
        if (!this.ngControl) {
            return false;
        }
        const isSubmitted = (this.parent && this.parent.submitted);
        return this.ngControl.invalid && (this.ngControl.touched || isSubmitted);
    }
    ngAfterViewInit() {
        const control = this.injector.get(NgControl, null);
        if (control) {
            this.ngControl = control.control;
            this.element.nativeElement.validationStrategy = this;
        }
        if (this.parent && this.parent.statusChanges) {
            this.parentSubscription = this.parent.statusChanges.subscribe({
                next: () => {
                    this.changeRef.markForCheck();
                }
            });
        }
    }
    ngOnDestroy() {
        if (this.parentSubscription) {
            this.parentSubscription.unsubscribe();
        }
    }
    onAnjBlur() {
        this.onTouched();
    }
    writeValue(value) {
        this.element.nativeElement.value = this.lastValue = value == null ? '' : value;
    }
    onChangeEvent(value) {
        if (value !== this.lastValue) {
            this.lastValue = value;
            this.onChange(value);
            this.valueChange.emit(value);
        }
    }
    registerOnChange(fn) {
        this.onChange = fn;
    }
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    setDisabledState(isDisabled) {
        this.element.nativeElement.disabled = isDisabled;
    }
    // Called by Anjuna Element, so needs to exist, but Angular does all the hoof-work
    onValidChange() { }
};
ValueAccessor.ctorParameters = () => [
    { type: ElementRef },
    { type: NgForm },
    { type: FormGroupDirective },
    { type: Injector },
    { type: ChangeDetectorRef, decorators: [{ type: Optional }] }
];
__decorate([
    HostBinding('attr.invalid'),
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], ValueAccessor.prototype, "isInvalid", null);
__decorate([
    Output(),
    __metadata("design:type", EventEmitter)
], ValueAccessor.prototype, "valueChange", void 0);
__decorate([
    HostListener('anjBlur'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ValueAccessor.prototype, "onAnjBlur", null);
ValueAccessor = __decorate([
    Directive(),
    __param(4, Optional()),
    __metadata("design:paramtypes", [ElementRef,
        NgForm,
        FormGroupDirective,
        Injector,
        ChangeDetectorRef])
], ValueAccessor);

var BooleanValueDirective_1;
/* tslint:disable:directive-selector */
let BooleanValueDirective = BooleanValueDirective_1 = class BooleanValueDirective extends ValueAccessor {
    constructor(parentForm, parentFormGroup, element, injector, changeRef) {
        super(element, parentForm, parentFormGroup, injector, changeRef);
    }
    writeValue(value) {
        this.element.nativeElement.checked = this.lastValue = value == null ? false : value;
    }
    onAnjChange(value) {
        this.onChangeEvent(value);
    }
};
BooleanValueDirective.ctorParameters = () => [
    { type: NgForm, decorators: [{ type: Optional }] },
    { type: FormGroupDirective, decorators: [{ type: Optional }] },
    { type: ElementRef },
    { type: Injector },
    { type: ChangeDetectorRef }
];
__decorate([
    HostListener('anjChange', ['$event.target.checked']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], BooleanValueDirective.prototype, "onAnjChange", null);
BooleanValueDirective = BooleanValueDirective_1 = __decorate([
    Directive({
        selector: 'anj-checkbox, anj-toggle',
        providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: BooleanValueDirective_1,
                multi: true
            }
        ]
    }),
    __param(0, Optional()),
    __param(1, Optional()),
    __metadata("design:paramtypes", [NgForm,
        FormGroupDirective,
        ElementRef,
        Injector,
        ChangeDetectorRef])
], BooleanValueDirective);

/** @dynamic */
class AnjunaValidators {
    static date(c) {
        if (!c.value) {
            return;
        }
        let isValid = true;
        // Handle obvious Invalid Date
        if (c.value.toString() === 'Invalid Date') {
            isValid = false;
            // Handle Moment Date
        }
        else if (typeof c.value._isValid === 'boolean') {
            isValid = c.value._isValid;
            // Handle Native Date
        }
        else {
            isValid = !isNaN(new Date(c.value).getTime());
        }
        if (isValid || c.value === '') {
            return null;
        }
        else {
            return {
                date: {
                    valid: false
                }
            };
        }
    }
    static requiredCheck(c) {
        if (c.value) {
            return null;
        }
        else {
            return {
                required: {
                    valid: false
                }
            };
        }
    }
    static unique(c) {
        if (!c.value || !Array.isArray(c.value)) {
            return;
        }
        const testArray = c.value.filter(Boolean);
        const isValid = testArray.length === Array.from(new Set(testArray)).length;
        if (isValid) {
            return null;
        }
        else {
            return {
                unique: {
                    valid: false
                }
            };
        }
    }
}
AnjunaValidators.minDate = (min) => {
    return (c) => {
        if (!c.value) {
            return;
        }
        if (c.value < new Date(min)) {
            return {
                min: {
                    valid: false
                }
            };
        }
        else {
            return null;
        }
    };
};
AnjunaValidators.maxDate = (max) => {
    return (c) => {
        if (!c.value) {
            return;
        }
        if (c.value > new Date(max)) {
            return {
                max: {
                    valid: false
                }
            };
        }
        else {
            return null;
        }
    };
};

var DateValidatorDirective_1;
/* tslint:disable:directive-selector */
let DateValidatorDirective = DateValidatorDirective_1 = class DateValidatorDirective {
    validate(c) {
        return AnjunaValidators.date(c);
    }
};
DateValidatorDirective = DateValidatorDirective_1 = __decorate([
    Directive({
        selector: '[date][ngModel],[date][formControl]',
        providers: [
            { provide: NG_VALIDATORS, useExisting: DateValidatorDirective_1, multi: true }
        ]
    })
], DateValidatorDirective);

let PickerAbstract = class PickerAbstract {
    constructor(_element, changeRef, ngControl) {
        this._element = _element;
        this.changeRef = changeRef;
        this.ngControl = ngControl;
        this.onChange = () => { };
        this.onTouch = () => { };
        this.valueChange = new EventEmitter();
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
    }
    ngAfterViewInit() {
        this.picker = this._element.nativeElement.querySelector(this.pickerSelector);
        if (this.picker) {
            this.picker.setInputValue(this.innerDateValue);
            this.outerValue = this._element.nativeElement.value;
        }
    }
    // // Start: ControlValueAccessor requirement
    writeValue(newValue) {
        if (this.picker && this.innerDateValue !== newValue) {
            this.picker.setInputValue(newValue);
            this.outerValue = this._element.nativeElement.value;
        }
        this.innerDateValue = newValue;
    }
    registerOnChange(fn) {
        this.onChange = fn;
    }
    registerOnTouched(fn) {
        this.onTouch = fn;
    }
    // // End: ControlValueAccessor requirements
    changeHandler(changedVal) {
        if (changedVal.refreshOuter) {
            this.outerValue = changedVal.outer;
        }
        this.innerDateValue = changedVal.inner;
        this.propogateInnerValueChange();
        this.changeRef.markForCheck();
        // Handle for keyup
        if (changedVal.inner) {
            this.onTouch();
        }
    }
    propogateInnerValueChange() {
        if (this.ngControl) {
            this.ngControl.control.setValue(this.innerDateValue);
        }
        this.valueChange.emit(this.innerDateValue);
    }
};
PickerAbstract.ctorParameters = () => [
    { type: ElementRef },
    { type: ChangeDetectorRef },
    { type: NgControl, decorators: [{ type: Optional }, { type: Self }] }
];
__decorate([
    Input(),
    __metadata("design:type", NgModel)
], PickerAbstract.prototype, "ngModel", void 0);
__decorate([
    HostBinding('value'),
    __metadata("design:type", String)
], PickerAbstract.prototype, "outerValue", void 0);
PickerAbstract = __decorate([
    Directive(),
    __param(2, Optional()), __param(2, Self()),
    __metadata("design:paramtypes", [ElementRef,
        ChangeDetectorRef,
        NgControl])
], PickerAbstract);

let DatePickerDirective = class DatePickerDirective extends PickerAbstract {
    constructor(_element, changeRef, ngControl) {
        super(_element, changeRef, ngControl);
        this.pickerSelector = 'anj-date-picker';
    }
    onAnjBlur() {
        this.onTouch();
    }
    onAnjDateChange(event) {
        this.changeHandler(event.detail);
    }
};
DatePickerDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: ChangeDetectorRef },
    { type: NgControl, decorators: [{ type: Optional }, { type: Self }] }
];
__decorate([
    HostListener('anjBlur'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], DatePickerDirective.prototype, "onAnjBlur", null);
__decorate([
    HostListener('anjDateChange', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], DatePickerDirective.prototype, "onAnjDateChange", null);
DatePickerDirective = __decorate([
    Directive({
        selector: 'anj-input[anjDatePicker]',
        exportAs: 'anjDatepickerInput'
    }),
    __param(2, Optional()), __param(2, Self()),
    __metadata("design:paramtypes", [ElementRef,
        ChangeDetectorRef,
        NgControl])
], DatePickerDirective);

var TextValueDirective_1;
/* tslint:disable:directive-selector */
let TextValueDirective = TextValueDirective_1 = class TextValueDirective extends ValueAccessor {
    constructor(parentForm, parentFormGroup, element, injector, changeRef) {
        super(element, parentForm, parentFormGroup, injector, changeRef);
    }
    onAnjChange(value) {
        this.onChangeEvent(value);
    }
};
TextValueDirective.ctorParameters = () => [
    { type: NgForm, decorators: [{ type: Optional }] },
    { type: FormGroupDirective, decorators: [{ type: Optional }] },
    { type: ElementRef },
    { type: Injector },
    { type: ChangeDetectorRef }
];
__decorate([
    HostListener('anjChange', ['$event.detail.value']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], TextValueDirective.prototype, "onAnjChange", null);
TextValueDirective = TextValueDirective_1 = __decorate([
    Directive({
        selector: 'anj-input, anj-textarea',
        providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: TextValueDirective_1,
                multi: true
            }
        ]
    }),
    __param(0, Optional()),
    __param(1, Optional()),
    __metadata("design:paramtypes", [NgForm,
        FormGroupDirective,
        ElementRef,
        Injector,
        ChangeDetectorRef])
], TextValueDirective);

var RequiredCheckDirective_1;
/* tslint:disable:directive-selector */
let RequiredCheckDirective = RequiredCheckDirective_1 = class RequiredCheckDirective {
    validate(c) {
        return AnjunaValidators.requiredCheck(c);
    }
};
RequiredCheckDirective = RequiredCheckDirective_1 = __decorate([
    Directive({
        selector: 'anj-checkbox[required][ngModel], anj-toggle[required][ngModel]',
        providers: [
            { provide: NG_VALIDATORS, useExisting: RequiredCheckDirective_1, multi: true }
        ]
    })
], RequiredCheckDirective);

var SelectValueDirective_1;
/* tslint:disable:directive-selector */
let SelectValueDirective = SelectValueDirective_1 = class SelectValueDirective extends ValueAccessor {
    constructor(parentForm, parentFormGroup, element, injector, changeRef) {
        super(element, parentForm, parentFormGroup, injector, changeRef);
    }
    onAnjChange(target) {
        // ignore the input for type-aheads
        if (target.matches('anj-input')) {
            return;
        }
        this.onChangeEvent(target.value);
    }
};
SelectValueDirective.ctorParameters = () => [
    { type: NgForm, decorators: [{ type: Optional }] },
    { type: FormGroupDirective, decorators: [{ type: Optional }] },
    { type: ElementRef },
    { type: Injector },
    { type: ChangeDetectorRef }
];
__decorate([
    HostListener('anjChange', ['$event.target']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [HTMLInputElement]),
    __metadata("design:returntype", void 0)
], SelectValueDirective.prototype, "onAnjChange", null);
SelectValueDirective = SelectValueDirective_1 = __decorate([
    Directive({
        selector: 'anj-radio-group, anj-select, anj-type-ahead',
        providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: SelectValueDirective_1,
                multi: true
            }
        ]
    }),
    __param(0, Optional()),
    __param(1, Optional()),
    __metadata("design:paramtypes", [NgForm,
        FormGroupDirective,
        ElementRef,
        Injector,
        ChangeDetectorRef])
], SelectValueDirective);

let TimePickerDirective = class TimePickerDirective {
    constructor(_element, ngControl, _timezone, _meridiem) {
        this._element = _element;
        this.ngControl = ngControl;
        this._timezone = _timezone;
        this._meridiem = _meridiem;
        this.onChange = () => { };
        this.onTouch = () => { };
        this.valueChange = new EventEmitter();
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
    }
    ngAfterViewInit() {
        const picker = this._element.nativeElement.querySelector('anj-time-picker');
        picker.setAttribute('meridiem', this._meridiem);
        if (this._timezone) {
            picker.setAttribute('timezone', this._timezone);
        }
    }
    // // Start: ControlValueAccessor requirements
    writeValue(newValue) {
        this.value = newValue;
    }
    registerOnChange(fn) {
        this.onChange = fn;
    }
    registerOnTouched(fn) {
        this.onTouch = fn;
    }
    // // End: ControlValueAccessor requirements
    onInput(event) {
        this.value = event.detail;
        if (this.ngControl) {
            this.ngControl.control.setValue(this.value);
        }
        this.valueChange.emit(this.value);
    }
};
TimePickerDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: NgControl, decorators: [{ type: Optional }, { type: Self }] },
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [TIME_TIMEZONE_STRING,] }] },
    { type: Boolean, decorators: [{ type: Optional }, { type: Inject, args: [TIME_MERIDIEM,] }] }
];
__decorate([
    HostBinding('value'),
    __metadata("design:type", String)
], TimePickerDirective.prototype, "value", void 0);
__decorate([
    HostListener('anjClockSelection', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], TimePickerDirective.prototype, "onInput", null);
TimePickerDirective = __decorate([
    Directive({
        selector: 'anj-input[anjTimePicker]',
        exportAs: 'anjTimepickerInput'
    }),
    __param(1, Optional()), __param(1, Self()),
    __param(2, Optional()), __param(2, Inject(TIME_TIMEZONE_STRING)),
    __param(3, Optional()), __param(3, Inject(TIME_MERIDIEM)),
    __metadata("design:paramtypes", [ElementRef,
        NgControl, String, Boolean])
], TimePickerDirective);

let DatetimePickerDirective = class DatetimePickerDirective extends PickerAbstract {
    constructor(_element, changeRef, ngControl, _meridiem) {
        super(_element, changeRef, ngControl);
        this._meridiem = _meridiem;
        this.pickerSelector = 'anj-datetime-picker';
    }
    onAnjBlur() {
        this.onTouch();
    }
    onAnjDatetimeChange(event) {
        this.changeHandler(event.detail);
    }
    ngAfterViewInit() {
        super.ngAfterViewInit();
        this.picker.setAttribute('meridiem', this._meridiem);
    }
};
DatetimePickerDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: ChangeDetectorRef },
    { type: NgControl, decorators: [{ type: Optional }, { type: Self }] },
    { type: Boolean, decorators: [{ type: Optional }, { type: Inject, args: [TIME_MERIDIEM,] }] }
];
__decorate([
    HostListener('anjBlur'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], DatetimePickerDirective.prototype, "onAnjBlur", null);
__decorate([
    HostListener('anjDatetimeChange', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], DatetimePickerDirective.prototype, "onAnjDatetimeChange", null);
DatetimePickerDirective = __decorate([
    Directive({
        selector: 'anj-input[anjDatetimePicker]',
        exportAs: 'anjDatetimePickerInput'
    }),
    __param(2, Optional()), __param(2, Self()),
    __param(3, Optional()), __param(3, Inject(TIME_MERIDIEM)),
    __metadata("design:paramtypes", [ElementRef,
        ChangeDetectorRef,
        NgControl, Boolean])
], DatetimePickerDirective);

/* tslint:disable:directive-selector */
let ListValueDirective = class ListValueDirective {
    constructor(_element, changeRef, parentForm, parentGroup, ngControl) {
        this._element = _element;
        this.changeRef = changeRef;
        this.ngControl = ngControl;
        this.valueChange = new EventEmitter();
        this.onChange = () => { };
        this.onTouch = () => { };
        this.parent = parentForm ? parentForm : parentGroup ? parentGroup : null;
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
    }
    get parsedField() {
        return (this.field) ? this.field.parsedList : undefined;
    }
    get list() {
        if (!this._list) {
            this.list = this._element.nativeElement;
        }
        return this._list;
    }
    set list(el) {
        this._list = el;
    }
    onAnjBlur() {
        this.onTouch();
    }
    onAnjChange(target) {
        if (!target.matches('anj-list')) {
            return;
        }
        this.changeHandler(target.value.slice(0));
    }
    get isInvalid() {
        if (this.ngControl) {
            const isSubmitted = (this.parent && this.parent.submitted);
            return (this.ngControl.invalid && (this.ngControl.touched || isSubmitted));
        }
        if (this.control) {
            return this.control.invalid;
        }
    }
    ngAfterViewInit() {
        if ((this.ngControl && this.ngControl.value) || this.ngModel) {
            this.value = this.ngModel || this.ngControl.value;
            this.list.value = this.value;
            this.changeRef.detectChanges();
        }
    }
    ngOnChanges() {
        this.setAttributes();
        if (this.control && this.control.value) {
            this.value = this.control.value;
            this.list.value = this.value;
            this.changeRef.detectChanges();
        }
    }
    setAttributes() {
        if (this.field) {
            this.list.setAttribute('min', this.field.minItems || this.list.min);
            this.list.setAttribute('max', this.field.maxItems || this.list.max);
        }
        if (this.parsedField) {
            this.list.setAttribute('options', this.parsedField.options || this.list.options);
            this.list.setAttribute('type', this.parsedField.toInputType() || this.list.type);
        }
    }
    // Start: ControlValueAccessor requirements
    writeValue(newValue) {
        this.value = newValue;
        this.list.value = this.value;
    }
    registerOnChange(fn) {
        this.onChange = fn;
    }
    registerOnTouched(fn) {
        this.onTouch = fn;
    }
    // End: ControlValueAccessor requirements
    changeHandler(changedVal) {
        this.onTouch();
        this.value = changedVal;
        this.valueChange.emit(this.value);
        this.propogateValueChange();
    }
    propogateValueChange() {
        if (this.ngControl) {
            this.ngControl.control.patchValue(this.value);
        }
        else if (this.control) {
            this.control.controls = [];
            this.value.map(value => this.control.push(new FormControl(value)));
        }
        this.changeRef.detectChanges();
    }
};
ListValueDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: ChangeDetectorRef },
    { type: NgForm, decorators: [{ type: Optional }] },
    { type: FormGroupDirective, decorators: [{ type: Optional }] },
    { type: NgControl, decorators: [{ type: Optional }, { type: Self }] }
];
__decorate([
    Input(),
    __metadata("design:type", NgModel)
], ListValueDirective.prototype, "ngModel", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], ListValueDirective.prototype, "control", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], ListValueDirective.prototype, "field", void 0);
__decorate([
    Output(),
    __metadata("design:type", EventEmitter)
], ListValueDirective.prototype, "valueChange", void 0);
__decorate([
    HostListener('anjBlur'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ListValueDirective.prototype, "onAnjBlur", null);
__decorate([
    HostListener('anjChange', ['$event.target']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [HTMLInputElement]),
    __metadata("design:returntype", void 0)
], ListValueDirective.prototype, "onAnjChange", null);
__decorate([
    HostBinding('attr.invalid'),
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], ListValueDirective.prototype, "isInvalid", null);
ListValueDirective = __decorate([
    Directive({
        selector: 'anj-list'
    }),
    __param(2, Optional()),
    __param(3, Optional()),
    __param(4, Optional()), __param(4, Self()),
    __metadata("design:paramtypes", [ElementRef,
        ChangeDetectorRef,
        NgForm,
        FormGroupDirective,
        NgControl])
], ListValueDirective);

let ZuoraDatePipe = class ZuoraDatePipe {
    constructor(_locale) {
        this._locale = _locale;
    }
    transform(value, args) {
        try {
            const valueAsDate = this.convertToDate(value);
            const userOpts = args;
            const defaultOpts = { year: '2-digit', month: 'numeric', day: 'numeric' };
            const opts = Object.assign({}, defaultOpts, userOpts);
            return new Intl.DateTimeFormat(this._locale, opts).format(valueAsDate);
        }
        catch (e) {
            return value.toString();
        }
    }
    /*
        Sometimes JS is awful, handles for 'YYYY-MM-DD' to choose local timezone instead of UTC
        https://stackoverflow.com/questions/2587345/why-does-date-parse-give-incorrect-results
     */
    convertToDate(input) {
        if (this.isIsoUtc(input)) {
            const splitString = input.split('-');
            return new Date(Number(splitString[0]), Number(splitString[1]) - 1, Number(splitString[2]));
        }
        else {
            return new Date(input);
        }
    }
    isIsoUtc(input) {
        return typeof input === 'string' && input.charAt(4) === '-' && input.charAt(7) === '-' && input.length === 10;
    }
};
ZuoraDatePipe.ctorParameters = () => [
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LOCALE_ID,] }] }
];
ZuoraDatePipe = __decorate([
    Pipe({
        name: 'zuoraDate'
    }),
    __param(0, Optional()), __param(0, Inject(LOCALE_ID)),
    __metadata("design:paramtypes", [String])
], ZuoraDatePipe);

var ToastService_1;
let ToastService = ToastService_1 = class ToastService {
    constructor() {
        this._dismissed$ = new Subject();
    }
    get dismissed$() {
        return this._dismissed$.asObservable();
    }
    static createController() {
        const controller = document.createElement('anj-toast-controller');
        document.body.appendChild(controller);
        return controller;
    }
    dismissAll() {
        return __awaiter(this, void 0, void 0, function* () {
            const controller = yield this.getController();
            return controller.dismissAll();
        });
    }
    error(message, header) {
        return __awaiter(this, void 0, void 0, function* () {
            const controller = yield this.getController();
            return controller.error(message, header);
        });
    }
    message(message, header) {
        return __awaiter(this, void 0, void 0, function* () {
            const controller = yield this.getController();
            return controller.message(message, header);
        });
    }
    success(message, header) {
        return __awaiter(this, void 0, void 0, function* () {
            const controller = yield this.getController();
            return controller.success(message, header);
        });
    }
    warn(message, header) {
        return __awaiter(this, void 0, void 0, function* () {
            const controller = yield this.getController();
            return controller.warn(message, header);
        });
    }
    getController() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.controller || this.findController();
        });
    }
    findController() {
        const controller = this.findOrCreate(document.querySelector('anj-toast-controller'));
        controller.addEventListener('anjToastDidDismiss', this.onToastDidDismiss.bind(this));
        this.controller = controller;
        return controller.componentOnReady();
    }
    onToastDidDismiss(event) {
        this._dismissed$.next(event);
    }
    findOrCreate(control) {
        return control || ToastService_1.createController();
    }
};
ToastService.ɵprov = ɵɵdefineInjectable({ factory: function ToastService_Factory() { return new ToastService(); }, token: ToastService, providedIn: "root" });
ToastService = ToastService_1 = __decorate([
    Injectable({ providedIn: 'root' })
], ToastService);

let LocalStorageService = class LocalStorageService extends LocalStorage {
    constructor() {
        super();
    }
};
LocalStorageService.ɵprov = ɵɵdefineInjectable({ factory: function LocalStorageService_Factory() { return new LocalStorageService(); }, token: LocalStorageService, providedIn: "root" });
LocalStorageService = __decorate([
    Injectable({ providedIn: 'root' }),
    __metadata("design:paramtypes", [])
], LocalStorageService);
let SessionStorageService = class SessionStorageService extends SessionStorage {
    constructor() {
        super();
    }
};
SessionStorageService.ɵprov = ɵɵdefineInjectable({ factory: function SessionStorageService_Factory() { return new SessionStorageService(); }, token: SessionStorageService, providedIn: "root" });
SessionStorageService = __decorate([
    Injectable({ providedIn: 'root' }),
    __metadata("design:paramtypes", [])
], SessionStorageService);

const DECLARATIONS = [
    // Common
    DraggableDirective,
    DroppableDirective,
    LocaleDirective,
    TemplateDirective,
    TransitionDirective,
    ZuoraDatePipe,
    // Forms
    BooleanValueDirective,
    DatePickerDirective,
    DatetimePickerDirective,
    DateValidatorDirective,
    ListValueDirective,
    RequiredCheckDirective,
    SelectValueDirective,
    TextValueDirective,
    TimePickerDirective,
    // Tables
    AnjTable
];
let AnjunaCoreModule = class AnjunaCoreModule {
};
AnjunaCoreModule = __decorate([
    NgModule({
        declarations: DECLARATIONS,
        exports: DECLARATIONS,
        providers: [
            {
                provide: TIME_MERIDIEM,
                useValue: true
            },
            ZuoraDatePipe
        ]
    })
], AnjunaCoreModule);

/* Abstracts */

/**
 * Generated bundle index. Do not edit.
 */

export { AnjTable, AnjunaCoreModule, AnjunaValidators, BaseTranslationPipe, BooleanValueDirective, DATE_TIMEZONE_OVERRIDE, DatePickerDirective, DateValidatorDirective, DatetimePickerDirective, DomOutlet, DraggableDirective, DroppableDirective, LANGUAGE_CODE, ListValueDirective, LocalStorageService, LocaleDirective, RequiredCheckDirective, SelectValueDirective, SessionStorageService, TIME_MERIDIEM, TIME_TIMEZONE_STRING, Template, TemplateDirective, TextValueDirective, TimePickerDirective, ToastService, TransitionDirective, TransitionState, ZuoraDatePipe, ValueAccessor as ɵa, PickerAbstract as ɵb };
//# sourceMappingURL=anjuna-angular-core.js.map
