import { __decorate, __param, __metadata, __extends, __awaiter, __generator } from 'tslib';
import { InjectionToken, Optional, Inject, ElementRef, Input, Directive, TemplateRef, ViewContainerRef, NgZone, ContentChildren, QueryList, HostListener, EventEmitter, Renderer2, Attribute, Output, ChangeDetectorRef, LOCALE_ID, HostBinding, Injector, Self, Pipe, ɵɵdefineInjectable, Injectable, NgModule } from '@angular/core';
import { DragController, DropController, LocalStorage, SessionStorage } from '@anjuna/core';
import { NgControl, NgForm, FormGroupDirective, NG_VALUE_ACCESSOR, NG_VALIDATORS, NgModel, FormControl } from '@angular/forms';
import { Subject } from 'rxjs';

var LANGUAGE_CODE = new InjectionToken('LanguageCode');

var BaseTranslationPipe = /** @class */ (function () {
    function BaseTranslationPipe(language) {
        this.language = language;
    }
    BaseTranslationPipe.prototype.transform = function (value) {
        if (!this.translations[this.language]) {
            console.warn("Translation file for '" + this.language + "' does not exist.");
            return value;
        }
        return this.translations[this.language][value] || value;
    };
    BaseTranslationPipe = __decorate([
        __param(0, Optional()), __param(0, Inject(LANGUAGE_CODE)),
        __metadata("design:paramtypes", [Object])
    ], BaseTranslationPipe);
    return BaseTranslationPipe;
}());

/**
 * @description A wrapper around the HTML 5 Drag API.
 */
var DraggableDirective = /** @class */ (function () {
    function DraggableDirective(element) {
        var _this = this;
        this.element = element;
        this.dataFn = function () { return _this.getData(); };
        this.controller = new DragController(this.element.nativeElement, this.dataFn).connect();
    }
    DraggableDirective.prototype.getData = function () {
        return this.anjDraggable;
    };
    DraggableDirective.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], DraggableDirective.prototype, "anjDraggable", void 0);
    DraggableDirective = __decorate([
        Directive({
            selector: '[anjDraggable]'
        }),
        __metadata("design:paramtypes", [ElementRef])
    ], DraggableDirective);
    return DraggableDirective;
}());

/**
 * @description A wrapper around the HTML 5 Drop API.
 */
var DroppableDirective = /** @class */ (function () {
    function DroppableDirective(element) {
        this.element = element;
        this.controller = new DropController(this.element.nativeElement).connect();
    }
    DroppableDirective.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    DroppableDirective = __decorate([
        Directive({
            selector: '[anjDroppable]'
        }),
        __metadata("design:paramtypes", [ElementRef])
    ], DroppableDirective);
    return DroppableDirective;
}());

var DomOutlet = /** @class */ (function () {
    function DomOutlet(targetElement) {
        this.targetElement = targetElement;
    }
    DomOutlet.prototype.attachTemplate = function (template) {
        var _this = this;
        if (this.template.zone.isStable) {
            this._viewRef = this.template.zone.run(function () {
                return template.viewContainerRef.createEmbeddedView(template.templateRef, template.context);
            });
        }
        else {
            this._viewRef = template.viewContainerRef.createEmbeddedView(template.templateRef, template.context);
        }
        this._viewRef.rootNodes.forEach(function (rootNode) { return _this.targetElement.appendChild(rootNode); });
        this._viewRef.detectChanges();
        this.setDisposeFn((function () {
            var index = template.viewContainerRef.indexOf(_this._viewRef);
            if (index !== -1) {
                template.viewContainerRef.remove(index);
            }
        }));
        return this._viewRef;
    };
    DomOutlet.prototype.updateContext = function (context) {
        for (var prop in context) {
            if (context.hasOwnProperty(prop)) {
                this._viewRef.context[prop] = context[prop];
            }
        }
        this._viewRef.detectChanges();
    };
    DomOutlet.prototype.setDisposeFn = function (fn) {
        this.disposeFn = fn;
    };
    DomOutlet.prototype.attach = function (template) {
        this.template = template;
        return this.attachTemplate(template);
    };
    DomOutlet.prototype.detach = function () {
        if (this.template) {
            this.template.setAttachedHost(null);
            this.template = null;
        }
        this.invokeDisposeFn();
    };
    DomOutlet.prototype.dispose = function (removeParent) {
        if (removeParent === void 0) { removeParent = true; }
        this.disposeFn();
        if (removeParent && this.targetElement.parentNode != null) {
            this.targetElement.parentNode.removeChild(this.targetElement);
        }
        return undefined;
    };
    DomOutlet.prototype.invokeDisposeFn = function () {
        if (this.disposeFn) {
            this.disposeFn();
            this.disposeFn = null;
        }
    };
    return DomOutlet;
}());

var Template = /** @class */ (function () {
    function Template(templateDirective, context) {
        this.templateDirective = templateDirective;
        this.context = context;
    }
    Object.defineProperty(Template.prototype, "origin", {
        get: function () {
            return this.templateDirective.template.elementRef;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Template.prototype, "templateRef", {
        get: function () {
            return this.templateDirective.template;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Template.prototype, "viewContainerRef", {
        get: function () {
            return this.templateDirective.viewContainerRef;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Template.prototype, "zone", {
        get: function () {
            return this.templateDirective.zone;
        },
        enumerable: true,
        configurable: true
    });
    Template.prototype.attach = function (host, context) {
        if (context === void 0) { context = this.context; }
        this.context = context;
        this.host = host;
        return host.attach(this);
    };
    Template.prototype.detach = function () {
        this.context = undefined;
        if (this.host) {
            var host = this.host;
            this.host = null;
            host.detach();
        }
    };
    Template.prototype.setAttachedHost = function (host) {
        this.host = host;
    };
    return Template;
}());

var TemplateDirective = /** @class */ (function () {
    function TemplateDirective(_template, _viewRef, _zone) {
        this._template = _template;
        this._viewRef = _viewRef;
        this._zone = _zone;
    }
    Object.defineProperty(TemplateDirective.prototype, "anjTemplateOf", {
        set: function (name) {
            this._name = name;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TemplateDirective.prototype, "name", {
        get: function () {
            return this._name;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TemplateDirective.prototype, "template", {
        get: function () {
            return this._template;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TemplateDirective.prototype, "viewContainerRef", {
        get: function () {
            return this._viewRef;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TemplateDirective.prototype, "zone", {
        get: function () {
            return this._zone;
        },
        enumerable: true,
        configurable: true
    });
    TemplateDirective.ctorParameters = function () { return [
        { type: TemplateRef },
        { type: ViewContainerRef },
        { type: NgZone }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", String),
        __metadata("design:paramtypes", [String])
    ], TemplateDirective.prototype, "anjTemplateOf", null);
    TemplateDirective = __decorate([
        Directive({
            selector: '[anjTemplate]'
        }),
        __metadata("design:paramtypes", [TemplateRef, ViewContainerRef, NgZone])
    ], TemplateDirective);
    return TemplateDirective;
}());

/* tslint:disable:directive-selector */
var AnjTable = /** @class */ (function () {
    function AnjTable(elRef) {
        this.outletRows = [];
        this.element = elRef.nativeElement;
        this.element.renderer = this;
    }
    AnjTable_1 = AnjTable;
    AnjTable.rowContext = function (row, index, column) {
        return {
            $implicit: row,
            column: column,
            index: index
        };
    };
    AnjTable.updateContext = function (outlet, context) {
        outlet.updateContext(context);
    };
    AnjTable.prototype.ngOnDestroy = function () {
        this.outletRows.map(function (rowMap) {
            for (var templateName in rowMap) {
                if (rowMap[templateName]) {
                    rowMap[templateName] = rowMap[templateName].dispose();
                }
            }
            return rowMap;
        });
    };
    /**
     * Fixes https://git.zias.io/ui-foundation/anjuna/issues/876
     * When the columns change, destory all portals without killing the parent node
     */
    AnjTable.prototype.onColumnsUpdated = function () {
        this.outletRows.map(function (rowMap) {
            for (var templateName in rowMap) {
                if (rowMap[templateName]) {
                    rowMap[templateName] = rowMap[templateName].dispose(false);
                }
            }
            return rowMap;
        });
    };
    /* These next three methods are a mess of duplicated logic - they will probably stay that way until feature complete, then I'll clean em up */
    AnjTable.prototype.cellTemplate = function (cell, offset, col, row, rowIndex, template) {
        var rowMap = this.outletRows[rowIndex] || {};
        var outlet = rowMap[col.name];
        if (!cell) {
            /**
             * Removing the line:
             * rowMap[col.name] = outlet ? outlet.dispose() : undefined;
             * which fixes https://git.zias.io/ui-foundation/anjuna/issues/850.
             * Like the detail situation, I do not understand why this works..
             */
            return;
        }
        if (outlet) {
            if (cell !== outlet.targetElement) {
                this.updateOutlet(outlet, template, cell, AnjTable_1.rowContext(row, rowIndex, col));
            }
            else {
                AnjTable_1.updateContext(outlet, AnjTable_1.rowContext(row, rowIndex, col));
            }
        }
        else {
            rowMap[col.name] = this.createOutlet(cell, template, AnjTable_1.rowContext(row, rowIndex, col));
            this.outletRows[rowIndex] = rowMap;
        }
        col.initCell(cell, offset);
    };
    AnjTable.prototype.rowTemplate = function (templateName, targetEl, row, rowIndex) {
        // Dispose row template no longer used for that index
        this.outletRows[rowIndex] = this.filterRowTemplates(this.outletRows[rowIndex], templateName);
        var outlet = this.outletRows[rowIndex][templateName];
        if (!targetEl) {
            this.outletRows[rowIndex][templateName] = outlet ? outlet.dispose() : undefined;
            return;
        }
        if (outlet) {
            if (targetEl !== outlet.targetElement) {
                this.updateOutlet(outlet, templateName, targetEl, AnjTable_1.rowContext(row, rowIndex));
            }
            else {
                AnjTable_1.updateContext(outlet, AnjTable_1.rowContext(row, rowIndex));
            }
        }
        else {
            this.outletRows[rowIndex][templateName] = this.createOutlet(targetEl, templateName, AnjTable_1.rowContext(row, rowIndex));
            this.outletRows[rowIndex] = this.outletRows[rowIndex];
        }
    };
    AnjTable.prototype.fromTemplate = function (templateName, targetEl, row, rowIndex) {
        var rowMap = this.outletRows[rowIndex] || {};
        var outlet = rowMap[templateName];
        if (!targetEl) {
            /**
             * Removing the line:
             * rowMap[templateName] = outlet ? outlet.dispose() : undefined;
             * is part of the fix for https://git.zias.io/ui-foundation/anjuna/issues/750
             * However, I don't grok at all how/why this works. I don't buy it.
             * So when we eventually get an issue related to master detail, start here.
             * Put that line back, and fix it right.
             */
            return;
        }
        if (outlet) {
            if (targetEl !== outlet.targetElement) {
                this.updateOutlet(outlet, templateName, targetEl, AnjTable_1.rowContext(row, rowIndex));
            }
            else {
                AnjTable_1.updateContext(outlet, AnjTable_1.rowContext(row, rowIndex));
            }
        }
        else {
            rowMap[templateName] = this.createOutlet(targetEl, templateName, AnjTable_1.rowContext(row, rowIndex));
            this.outletRows[rowIndex] = rowMap;
        }
    };
    AnjTable.prototype.createOutlet = function (target, templateName, context) {
        var outlet = new DomOutlet(target);
        outlet.attach(this.createTemplate(templateName, context));
        return outlet;
    };
    AnjTable.prototype.createTemplate = function (templateName, context) {
        var templateRef = this.templates.find(function (t) { return t.name === templateName; });
        return new Template(templateRef, context);
    };
    AnjTable.prototype.updateOutlet = function (outlet, templateName, targetEl, context) {
        outlet.targetElement = targetEl;
        outlet.detach();
        outlet.attach(this.createTemplate(templateName, context));
    };
    /**
     * Disposes of row templates no longer used in a given row. This ONLY happens if they have specified more than one
     * Template and the end user sorts or filters or otherwise changes the order. If only one template is specified,
     * the output will always match the input -r
     */
    AnjTable.prototype.filterRowTemplates = function (rowMap, templateName) {
        if (rowMap === void 0) { rowMap = {}; }
        var updated = {};
        Object.keys(rowMap).forEach(function (k) {
            if (k === templateName) {
                updated[k] = rowMap[k];
            }
            else {
                rowMap[k].dispose(false);
            }
        });
        return updated;
    };
    var AnjTable_1;
    AnjTable.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    __decorate([
        ContentChildren(TemplateDirective),
        __metadata("design:type", QueryList)
    ], AnjTable.prototype, "templates", void 0);
    __decorate([
        HostListener('anjColumnsDidUpdate'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], AnjTable.prototype, "onColumnsUpdated", null);
    AnjTable = AnjTable_1 = __decorate([
        Directive({
            selector: 'anj-table:not([anjIgnore])',
        }),
        __metadata("design:paramtypes", [ElementRef])
    ], AnjTable);
    return AnjTable;
}());

var TransitionState;
(function (TransitionState) {
    TransitionState[TransitionState["Enter"] = 0] = "Enter";
    TransitionState[TransitionState["Leave"] = 1] = "Leave";
    TransitionState[TransitionState["EnterTo"] = 2] = "EnterTo";
    TransitionState[TransitionState["Inactive"] = 3] = "Inactive";
})(TransitionState || (TransitionState = {}));

var TransitionDirective = /** @class */ (function () {
    function TransitionDirective(renderer, element, mode, name) {
        this.renderer = renderer;
        this.element = element;
        this.mode = mode;
        this.name = name;
        this.state = TransitionState.EnterTo;
        this.end = new EventEmitter();
        this.mode = this.mode ? this.mode : 'in-out';
        this.name = this.name ? this.name : 'transition';
    }
    TransitionDirective_1 = TransitionDirective;
    TransitionDirective.prototype.ngOnChanges = function (changes) {
        this.endTransition();
        if (!changes.anjTransition.previousValue && !changes.anjTransition.currentValue && this.mode !== 'out') {
            this.initEnter();
        }
        else if (!changes.anjTransition.previousValue && changes.anjTransition.currentValue) {
            this.transitionIn();
        }
        else {
            this.transitionOut();
        }
    };
    TransitionDirective.prototype.handleTransition = function (event) {
        if (event.target === this.element.nativeElement && TransitionDirective_1.supportedTransitions.indexOf(event.propertyName) > -1) {
            event.stopPropagation();
            this.end.emit(this.state);
            if (this.state === TransitionState.Enter) {
                this.endTransition();
            }
            else {
                if (this.mode !== 'out' && this.anjTransition) {
                    this.transitionIn();
                }
                else {
                    this.endTransition();
                }
            }
        }
    };
    TransitionDirective.prototype.initEnter = function () {
        this.renderer.addClass(this.element.nativeElement, "anj-" + this.name + "-enter-to");
    };
    TransitionDirective.prototype.transitionIn = function () {
        if (this.mode !== 'out') {
            this.state = TransitionState.Enter;
            this.renderer.removeClass(this.element.nativeElement, "anj-" + this.name + "-enter-to");
            this.renderer.removeClass(this.element.nativeElement, "anj-" + this.name + "-leave");
            this.renderer.removeClass(this.element.nativeElement, "anj-" + this.name + "-leave-active");
            this.renderer.addClass(this.element.nativeElement, "anj-" + this.name + "-enter-active");
            this.renderer.addClass(this.element.nativeElement, "anj-" + this.name + "-enter");
        }
        else {
            this.renderer.removeClass(this.element.nativeElement, "anj-" + this.name + "-enter-to");
            this.endTransition();
        }
    };
    TransitionDirective.prototype.transitionOut = function () {
        var _this = this;
        this.state = TransitionState.Leave;
        if (this.mode !== 'in') {
            this.renderer.addClass(this.element.nativeElement, "anj-" + this.name + "-leave-active");
            this.renderer.addClass(this.element.nativeElement, "anj-" + this.name + "-leave");
        }
        else {
            this.renderer.addClass(this.element.nativeElement, "anj-" + this.name + "-enter-to");
            this.state = TransitionState.EnterTo;
            if (this.anjTransition) {
                window.setTimeout(function () {
                    _this.transitionIn();
                });
            }
        }
    };
    TransitionDirective.prototype.endTransition = function () {
        this.state = TransitionState.Inactive;
        this.renderer.removeClass(this.element.nativeElement, "anj-" + this.name + "-enter-active");
        this.renderer.removeClass(this.element.nativeElement, "anj-" + this.name + "-leave-active");
        this.renderer.removeClass(this.element.nativeElement, "anj-" + this.name + "-enter");
        this.renderer.removeClass(this.element.nativeElement, "anj-" + this.name + "-leave");
        if (!this.anjTransition) {
            this.renderer.addClass(this.element.nativeElement, "anj-" + this.name + "-enter-to");
        }
    };
    var TransitionDirective_1;
    TransitionDirective.supportedTransitions = ['opacity', 'transform'];
    TransitionDirective.ctorParameters = function () { return [
        { type: Renderer2 },
        { type: ElementRef },
        { type: undefined, decorators: [{ type: Attribute, args: ['mode',] }] },
        { type: String, decorators: [{ type: Attribute, args: ['name',] }] }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], TransitionDirective.prototype, "anjTransition", void 0);
    __decorate([
        Output(),
        __metadata("design:type", EventEmitter)
    ], TransitionDirective.prototype, "end", void 0);
    __decorate([
        HostListener('transitiend', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [TransitionEvent]),
        __metadata("design:returntype", void 0)
    ], TransitionDirective.prototype, "handleTransition", null);
    TransitionDirective = TransitionDirective_1 = __decorate([
        Directive({
            selector: '[anjTransition]'
        }),
        __param(2, Attribute('mode')),
        __param(3, Attribute('name')),
        __metadata("design:paramtypes", [Renderer2,
            ElementRef, String, String])
    ], TransitionDirective);
    return TransitionDirective;
}());

var TIME_TIMEZONE_STRING = new InjectionToken('TimeTimezoneString');
var TIME_MERIDIEM = new InjectionToken('TimeMeridiem');
var DATE_TIMEZONE_OVERRIDE = new InjectionToken('DateTimezoneOverride');

/* tslint:disable:directive-selector */
var LocaleDirective = /** @class */ (function () {
    function LocaleDirective(changeRef, _locale, _lang, _timezone) {
        this.changeRef = changeRef;
        if (_locale) {
            this._locale = _locale;
            this._lang = _lang ? _lang : this._locale ? this._locale.split('-')[0] : 'en';
        }
        if (_timezone) {
            this._timezone = _timezone;
        }
        this.changeRef.markForCheck();
    }
    LocaleDirective.ctorParameters = function () { return [
        { type: ChangeDetectorRef },
        { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LOCALE_ID,] }] },
        { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LANGUAGE_CODE,] }] },
        { type: String, decorators: [{ type: Optional }, { type: Inject, args: [DATE_TIMEZONE_OVERRIDE,] }] }
    ]; };
    __decorate([
        HostBinding('attr.locale'),
        __metadata("design:type", String)
    ], LocaleDirective.prototype, "_locale", void 0);
    __decorate([
        HostBinding('attr.lang'),
        __metadata("design:type", String)
    ], LocaleDirective.prototype, "_lang", void 0);
    __decorate([
        HostBinding('attr.timezone'),
        __metadata("design:type", String)
    ], LocaleDirective.prototype, "_timezone", void 0);
    LocaleDirective = __decorate([
        Directive({
            selector: 'anj-locale:not([manual])',
        }),
        __param(1, Optional()), __param(1, Inject(LOCALE_ID)),
        __param(2, Optional()), __param(2, Inject(LANGUAGE_CODE)),
        __param(3, Optional()), __param(3, Inject(DATE_TIMEZONE_OVERRIDE)),
        __metadata("design:paramtypes", [ChangeDetectorRef, String, String, String])
    ], LocaleDirective);
    return LocaleDirective;
}());

var ValueAccessor = /** @class */ (function () {
    function ValueAccessor(element, parentForm, parentGroup, injector, changeRef) {
        this.element = element;
        this.injector = injector;
        this.changeRef = changeRef;
        /**
         * Enable two-way binding
         */
        this.valueChange = new EventEmitter();
        this.onChange = function () { };
        this.onTouched = function () { };
        this.parent = parentForm ? parentForm : parentGroup ? parentGroup : null;
    }
    Object.defineProperty(ValueAccessor.prototype, "isInvalid", {
        get: function () {
            if (!this.ngControl) {
                return false;
            }
            var isSubmitted = (this.parent && this.parent.submitted);
            return this.ngControl.invalid && (this.ngControl.touched || isSubmitted);
        },
        enumerable: true,
        configurable: true
    });
    ValueAccessor.prototype.ngAfterViewInit = function () {
        var _this = this;
        var control = this.injector.get(NgControl, null);
        if (control) {
            this.ngControl = control.control;
            this.element.nativeElement.validationStrategy = this;
        }
        if (this.parent && this.parent.statusChanges) {
            this.parentSubscription = this.parent.statusChanges.subscribe({
                next: function () {
                    _this.changeRef.markForCheck();
                }
            });
        }
    };
    ValueAccessor.prototype.ngOnDestroy = function () {
        if (this.parentSubscription) {
            this.parentSubscription.unsubscribe();
        }
    };
    ValueAccessor.prototype.onAnjBlur = function () {
        this.onTouched();
    };
    ValueAccessor.prototype.writeValue = function (value) {
        this.element.nativeElement.value = this.lastValue = value == null ? '' : value;
    };
    ValueAccessor.prototype.onChangeEvent = function (value) {
        if (value !== this.lastValue) {
            this.lastValue = value;
            this.onChange(value);
            this.valueChange.emit(value);
        }
    };
    ValueAccessor.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    ValueAccessor.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    ValueAccessor.prototype.setDisabledState = function (isDisabled) {
        this.element.nativeElement.disabled = isDisabled;
    };
    // Called by Anjuna Element, so needs to exist, but Angular does all the hoof-work
    ValueAccessor.prototype.onValidChange = function () { };
    ValueAccessor.ctorParameters = function () { return [
        { type: ElementRef },
        { type: NgForm },
        { type: FormGroupDirective },
        { type: Injector },
        { type: ChangeDetectorRef, decorators: [{ type: Optional }] }
    ]; };
    __decorate([
        HostBinding('attr.invalid'),
        __metadata("design:type", Boolean),
        __metadata("design:paramtypes", [])
    ], ValueAccessor.prototype, "isInvalid", null);
    __decorate([
        Output(),
        __metadata("design:type", EventEmitter)
    ], ValueAccessor.prototype, "valueChange", void 0);
    __decorate([
        HostListener('anjBlur'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], ValueAccessor.prototype, "onAnjBlur", null);
    ValueAccessor = __decorate([
        Directive(),
        __param(4, Optional()),
        __metadata("design:paramtypes", [ElementRef,
            NgForm,
            FormGroupDirective,
            Injector,
            ChangeDetectorRef])
    ], ValueAccessor);
    return ValueAccessor;
}());

/* tslint:disable:directive-selector */
var BooleanValueDirective = /** @class */ (function (_super) {
    __extends(BooleanValueDirective, _super);
    function BooleanValueDirective(parentForm, parentFormGroup, element, injector, changeRef) {
        return _super.call(this, element, parentForm, parentFormGroup, injector, changeRef) || this;
    }
    BooleanValueDirective_1 = BooleanValueDirective;
    BooleanValueDirective.prototype.writeValue = function (value) {
        this.element.nativeElement.checked = this.lastValue = value == null ? false : value;
    };
    BooleanValueDirective.prototype.onAnjChange = function (value) {
        this.onChangeEvent(value);
    };
    var BooleanValueDirective_1;
    BooleanValueDirective.ctorParameters = function () { return [
        { type: NgForm, decorators: [{ type: Optional }] },
        { type: FormGroupDirective, decorators: [{ type: Optional }] },
        { type: ElementRef },
        { type: Injector },
        { type: ChangeDetectorRef }
    ]; };
    __decorate([
        HostListener('anjChange', ['$event.target.checked']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], BooleanValueDirective.prototype, "onAnjChange", null);
    BooleanValueDirective = BooleanValueDirective_1 = __decorate([
        Directive({
            selector: 'anj-checkbox, anj-toggle',
            providers: [
                {
                    provide: NG_VALUE_ACCESSOR,
                    useExisting: BooleanValueDirective_1,
                    multi: true
                }
            ]
        }),
        __param(0, Optional()),
        __param(1, Optional()),
        __metadata("design:paramtypes", [NgForm,
            FormGroupDirective,
            ElementRef,
            Injector,
            ChangeDetectorRef])
    ], BooleanValueDirective);
    return BooleanValueDirective;
}(ValueAccessor));

/** @dynamic */
var AnjunaValidators = /** @class */ (function () {
    function AnjunaValidators() {
    }
    AnjunaValidators.date = function (c) {
        if (!c.value) {
            return;
        }
        var isValid = true;
        // Handle obvious Invalid Date
        if (c.value.toString() === 'Invalid Date') {
            isValid = false;
            // Handle Moment Date
        }
        else if (typeof c.value._isValid === 'boolean') {
            isValid = c.value._isValid;
            // Handle Native Date
        }
        else {
            isValid = !isNaN(new Date(c.value).getTime());
        }
        if (isValid || c.value === '') {
            return null;
        }
        else {
            return {
                date: {
                    valid: false
                }
            };
        }
    };
    AnjunaValidators.requiredCheck = function (c) {
        if (c.value) {
            return null;
        }
        else {
            return {
                required: {
                    valid: false
                }
            };
        }
    };
    AnjunaValidators.unique = function (c) {
        if (!c.value || !Array.isArray(c.value)) {
            return;
        }
        var testArray = c.value.filter(Boolean);
        var isValid = testArray.length === Array.from(new Set(testArray)).length;
        if (isValid) {
            return null;
        }
        else {
            return {
                unique: {
                    valid: false
                }
            };
        }
    };
    AnjunaValidators.minDate = function (min) {
        return function (c) {
            if (!c.value) {
                return;
            }
            if (c.value < new Date(min)) {
                return {
                    min: {
                        valid: false
                    }
                };
            }
            else {
                return null;
            }
        };
    };
    AnjunaValidators.maxDate = function (max) {
        return function (c) {
            if (!c.value) {
                return;
            }
            if (c.value > new Date(max)) {
                return {
                    max: {
                        valid: false
                    }
                };
            }
            else {
                return null;
            }
        };
    };
    return AnjunaValidators;
}());

/* tslint:disable:directive-selector */
var DateValidatorDirective = /** @class */ (function () {
    function DateValidatorDirective() {
    }
    DateValidatorDirective_1 = DateValidatorDirective;
    DateValidatorDirective.prototype.validate = function (c) {
        return AnjunaValidators.date(c);
    };
    var DateValidatorDirective_1;
    DateValidatorDirective = DateValidatorDirective_1 = __decorate([
        Directive({
            selector: '[date][ngModel],[date][formControl]',
            providers: [
                { provide: NG_VALIDATORS, useExisting: DateValidatorDirective_1, multi: true }
            ]
        })
    ], DateValidatorDirective);
    return DateValidatorDirective;
}());

var PickerAbstract = /** @class */ (function () {
    function PickerAbstract(_element, changeRef, ngControl) {
        this._element = _element;
        this.changeRef = changeRef;
        this.ngControl = ngControl;
        this.onChange = function () { };
        this.onTouch = function () { };
        this.valueChange = new EventEmitter();
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
    }
    PickerAbstract.prototype.ngAfterViewInit = function () {
        this.picker = this._element.nativeElement.querySelector(this.pickerSelector);
        if (this.picker) {
            this.picker.setInputValue(this.innerDateValue);
            this.outerValue = this._element.nativeElement.value;
        }
    };
    // // Start: ControlValueAccessor requirement
    PickerAbstract.prototype.writeValue = function (newValue) {
        if (this.picker && this.innerDateValue !== newValue) {
            this.picker.setInputValue(newValue);
            this.outerValue = this._element.nativeElement.value;
        }
        this.innerDateValue = newValue;
    };
    PickerAbstract.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    PickerAbstract.prototype.registerOnTouched = function (fn) {
        this.onTouch = fn;
    };
    // // End: ControlValueAccessor requirements
    PickerAbstract.prototype.changeHandler = function (changedVal) {
        if (changedVal.refreshOuter) {
            this.outerValue = changedVal.outer;
        }
        this.innerDateValue = changedVal.inner;
        this.propogateInnerValueChange();
        this.changeRef.markForCheck();
        // Handle for keyup
        if (changedVal.inner) {
            this.onTouch();
        }
    };
    PickerAbstract.prototype.propogateInnerValueChange = function () {
        if (this.ngControl) {
            this.ngControl.control.setValue(this.innerDateValue);
        }
        this.valueChange.emit(this.innerDateValue);
    };
    PickerAbstract.ctorParameters = function () { return [
        { type: ElementRef },
        { type: ChangeDetectorRef },
        { type: NgControl, decorators: [{ type: Optional }, { type: Self }] }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", NgModel)
    ], PickerAbstract.prototype, "ngModel", void 0);
    __decorate([
        HostBinding('value'),
        __metadata("design:type", String)
    ], PickerAbstract.prototype, "outerValue", void 0);
    PickerAbstract = __decorate([
        Directive(),
        __param(2, Optional()), __param(2, Self()),
        __metadata("design:paramtypes", [ElementRef,
            ChangeDetectorRef,
            NgControl])
    ], PickerAbstract);
    return PickerAbstract;
}());

var DatePickerDirective = /** @class */ (function (_super) {
    __extends(DatePickerDirective, _super);
    function DatePickerDirective(_element, changeRef, ngControl) {
        var _this = _super.call(this, _element, changeRef, ngControl) || this;
        _this.pickerSelector = 'anj-date-picker';
        return _this;
    }
    DatePickerDirective.prototype.onAnjBlur = function () {
        this.onTouch();
    };
    DatePickerDirective.prototype.onAnjDateChange = function (event) {
        this.changeHandler(event.detail);
    };
    DatePickerDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: ChangeDetectorRef },
        { type: NgControl, decorators: [{ type: Optional }, { type: Self }] }
    ]; };
    __decorate([
        HostListener('anjBlur'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], DatePickerDirective.prototype, "onAnjBlur", null);
    __decorate([
        HostListener('anjDateChange', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], DatePickerDirective.prototype, "onAnjDateChange", null);
    DatePickerDirective = __decorate([
        Directive({
            selector: 'anj-input[anjDatePicker]',
            exportAs: 'anjDatepickerInput'
        }),
        __param(2, Optional()), __param(2, Self()),
        __metadata("design:paramtypes", [ElementRef,
            ChangeDetectorRef,
            NgControl])
    ], DatePickerDirective);
    return DatePickerDirective;
}(PickerAbstract));

/* tslint:disable:directive-selector */
var TextValueDirective = /** @class */ (function (_super) {
    __extends(TextValueDirective, _super);
    function TextValueDirective(parentForm, parentFormGroup, element, injector, changeRef) {
        return _super.call(this, element, parentForm, parentFormGroup, injector, changeRef) || this;
    }
    TextValueDirective_1 = TextValueDirective;
    TextValueDirective.prototype.onAnjChange = function (value) {
        this.onChangeEvent(value);
    };
    var TextValueDirective_1;
    TextValueDirective.ctorParameters = function () { return [
        { type: NgForm, decorators: [{ type: Optional }] },
        { type: FormGroupDirective, decorators: [{ type: Optional }] },
        { type: ElementRef },
        { type: Injector },
        { type: ChangeDetectorRef }
    ]; };
    __decorate([
        HostListener('anjChange', ['$event.detail.value']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], TextValueDirective.prototype, "onAnjChange", null);
    TextValueDirective = TextValueDirective_1 = __decorate([
        Directive({
            selector: 'anj-input, anj-textarea',
            providers: [
                {
                    provide: NG_VALUE_ACCESSOR,
                    useExisting: TextValueDirective_1,
                    multi: true
                }
            ]
        }),
        __param(0, Optional()),
        __param(1, Optional()),
        __metadata("design:paramtypes", [NgForm,
            FormGroupDirective,
            ElementRef,
            Injector,
            ChangeDetectorRef])
    ], TextValueDirective);
    return TextValueDirective;
}(ValueAccessor));

/* tslint:disable:directive-selector */
var RequiredCheckDirective = /** @class */ (function () {
    function RequiredCheckDirective() {
    }
    RequiredCheckDirective_1 = RequiredCheckDirective;
    RequiredCheckDirective.prototype.validate = function (c) {
        return AnjunaValidators.requiredCheck(c);
    };
    var RequiredCheckDirective_1;
    RequiredCheckDirective = RequiredCheckDirective_1 = __decorate([
        Directive({
            selector: 'anj-checkbox[required][ngModel], anj-toggle[required][ngModel]',
            providers: [
                { provide: NG_VALIDATORS, useExisting: RequiredCheckDirective_1, multi: true }
            ]
        })
    ], RequiredCheckDirective);
    return RequiredCheckDirective;
}());

/* tslint:disable:directive-selector */
var SelectValueDirective = /** @class */ (function (_super) {
    __extends(SelectValueDirective, _super);
    function SelectValueDirective(parentForm, parentFormGroup, element, injector, changeRef) {
        return _super.call(this, element, parentForm, parentFormGroup, injector, changeRef) || this;
    }
    SelectValueDirective_1 = SelectValueDirective;
    SelectValueDirective.prototype.onAnjChange = function (target) {
        // ignore the input for type-aheads
        if (target.matches('anj-input')) {
            return;
        }
        this.onChangeEvent(target.value);
    };
    var SelectValueDirective_1;
    SelectValueDirective.ctorParameters = function () { return [
        { type: NgForm, decorators: [{ type: Optional }] },
        { type: FormGroupDirective, decorators: [{ type: Optional }] },
        { type: ElementRef },
        { type: Injector },
        { type: ChangeDetectorRef }
    ]; };
    __decorate([
        HostListener('anjChange', ['$event.target']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [HTMLInputElement]),
        __metadata("design:returntype", void 0)
    ], SelectValueDirective.prototype, "onAnjChange", null);
    SelectValueDirective = SelectValueDirective_1 = __decorate([
        Directive({
            selector: 'anj-radio-group, anj-select, anj-type-ahead',
            providers: [
                {
                    provide: NG_VALUE_ACCESSOR,
                    useExisting: SelectValueDirective_1,
                    multi: true
                }
            ]
        }),
        __param(0, Optional()),
        __param(1, Optional()),
        __metadata("design:paramtypes", [NgForm,
            FormGroupDirective,
            ElementRef,
            Injector,
            ChangeDetectorRef])
    ], SelectValueDirective);
    return SelectValueDirective;
}(ValueAccessor));

var TimePickerDirective = /** @class */ (function () {
    function TimePickerDirective(_element, ngControl, _timezone, _meridiem) {
        this._element = _element;
        this.ngControl = ngControl;
        this._timezone = _timezone;
        this._meridiem = _meridiem;
        this.onChange = function () { };
        this.onTouch = function () { };
        this.valueChange = new EventEmitter();
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
    }
    TimePickerDirective.prototype.ngAfterViewInit = function () {
        var picker = this._element.nativeElement.querySelector('anj-time-picker');
        picker.setAttribute('meridiem', this._meridiem);
        if (this._timezone) {
            picker.setAttribute('timezone', this._timezone);
        }
    };
    // // Start: ControlValueAccessor requirements
    TimePickerDirective.prototype.writeValue = function (newValue) {
        this.value = newValue;
    };
    TimePickerDirective.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    TimePickerDirective.prototype.registerOnTouched = function (fn) {
        this.onTouch = fn;
    };
    // // End: ControlValueAccessor requirements
    TimePickerDirective.prototype.onInput = function (event) {
        this.value = event.detail;
        if (this.ngControl) {
            this.ngControl.control.setValue(this.value);
        }
        this.valueChange.emit(this.value);
    };
    TimePickerDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: NgControl, decorators: [{ type: Optional }, { type: Self }] },
        { type: String, decorators: [{ type: Optional }, { type: Inject, args: [TIME_TIMEZONE_STRING,] }] },
        { type: Boolean, decorators: [{ type: Optional }, { type: Inject, args: [TIME_MERIDIEM,] }] }
    ]; };
    __decorate([
        HostBinding('value'),
        __metadata("design:type", String)
    ], TimePickerDirective.prototype, "value", void 0);
    __decorate([
        HostListener('anjClockSelection', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], TimePickerDirective.prototype, "onInput", null);
    TimePickerDirective = __decorate([
        Directive({
            selector: 'anj-input[anjTimePicker]',
            exportAs: 'anjTimepickerInput'
        }),
        __param(1, Optional()), __param(1, Self()),
        __param(2, Optional()), __param(2, Inject(TIME_TIMEZONE_STRING)),
        __param(3, Optional()), __param(3, Inject(TIME_MERIDIEM)),
        __metadata("design:paramtypes", [ElementRef,
            NgControl, String, Boolean])
    ], TimePickerDirective);
    return TimePickerDirective;
}());

var DatetimePickerDirective = /** @class */ (function (_super) {
    __extends(DatetimePickerDirective, _super);
    function DatetimePickerDirective(_element, changeRef, ngControl, _meridiem) {
        var _this = _super.call(this, _element, changeRef, ngControl) || this;
        _this._meridiem = _meridiem;
        _this.pickerSelector = 'anj-datetime-picker';
        return _this;
    }
    DatetimePickerDirective.prototype.onAnjBlur = function () {
        this.onTouch();
    };
    DatetimePickerDirective.prototype.onAnjDatetimeChange = function (event) {
        this.changeHandler(event.detail);
    };
    DatetimePickerDirective.prototype.ngAfterViewInit = function () {
        _super.prototype.ngAfterViewInit.call(this);
        this.picker.setAttribute('meridiem', this._meridiem);
    };
    DatetimePickerDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: ChangeDetectorRef },
        { type: NgControl, decorators: [{ type: Optional }, { type: Self }] },
        { type: Boolean, decorators: [{ type: Optional }, { type: Inject, args: [TIME_MERIDIEM,] }] }
    ]; };
    __decorate([
        HostListener('anjBlur'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], DatetimePickerDirective.prototype, "onAnjBlur", null);
    __decorate([
        HostListener('anjDatetimeChange', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], DatetimePickerDirective.prototype, "onAnjDatetimeChange", null);
    DatetimePickerDirective = __decorate([
        Directive({
            selector: 'anj-input[anjDatetimePicker]',
            exportAs: 'anjDatetimePickerInput'
        }),
        __param(2, Optional()), __param(2, Self()),
        __param(3, Optional()), __param(3, Inject(TIME_MERIDIEM)),
        __metadata("design:paramtypes", [ElementRef,
            ChangeDetectorRef,
            NgControl, Boolean])
    ], DatetimePickerDirective);
    return DatetimePickerDirective;
}(PickerAbstract));

/* tslint:disable:directive-selector */
var ListValueDirective = /** @class */ (function () {
    function ListValueDirective(_element, changeRef, parentForm, parentGroup, ngControl) {
        this._element = _element;
        this.changeRef = changeRef;
        this.ngControl = ngControl;
        this.valueChange = new EventEmitter();
        this.onChange = function () { };
        this.onTouch = function () { };
        this.parent = parentForm ? parentForm : parentGroup ? parentGroup : null;
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
    }
    Object.defineProperty(ListValueDirective.prototype, "parsedField", {
        get: function () {
            return (this.field) ? this.field.parsedList : undefined;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListValueDirective.prototype, "list", {
        get: function () {
            if (!this._list) {
                this.list = this._element.nativeElement;
            }
            return this._list;
        },
        set: function (el) {
            this._list = el;
        },
        enumerable: true,
        configurable: true
    });
    ListValueDirective.prototype.onAnjBlur = function () {
        this.onTouch();
    };
    ListValueDirective.prototype.onAnjChange = function (target) {
        if (!target.matches('anj-list')) {
            return;
        }
        this.changeHandler(target.value.slice(0));
    };
    Object.defineProperty(ListValueDirective.prototype, "isInvalid", {
        get: function () {
            if (this.ngControl) {
                var isSubmitted = (this.parent && this.parent.submitted);
                return (this.ngControl.invalid && (this.ngControl.touched || isSubmitted));
            }
            if (this.control) {
                return this.control.invalid;
            }
        },
        enumerable: true,
        configurable: true
    });
    ListValueDirective.prototype.ngAfterViewInit = function () {
        if ((this.ngControl && this.ngControl.value) || this.ngModel) {
            this.value = this.ngModel || this.ngControl.value;
            this.list.value = this.value;
            this.changeRef.detectChanges();
        }
    };
    ListValueDirective.prototype.ngOnChanges = function () {
        this.setAttributes();
        if (this.control && this.control.value) {
            this.value = this.control.value;
            this.list.value = this.value;
            this.changeRef.detectChanges();
        }
    };
    ListValueDirective.prototype.setAttributes = function () {
        if (this.field) {
            this.list.setAttribute('min', this.field.minItems || this.list.min);
            this.list.setAttribute('max', this.field.maxItems || this.list.max);
        }
        if (this.parsedField) {
            this.list.setAttribute('options', this.parsedField.options || this.list.options);
            this.list.setAttribute('type', this.parsedField.toInputType() || this.list.type);
        }
    };
    // Start: ControlValueAccessor requirements
    ListValueDirective.prototype.writeValue = function (newValue) {
        this.value = newValue;
        this.list.value = this.value;
    };
    ListValueDirective.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    ListValueDirective.prototype.registerOnTouched = function (fn) {
        this.onTouch = fn;
    };
    // End: ControlValueAccessor requirements
    ListValueDirective.prototype.changeHandler = function (changedVal) {
        this.onTouch();
        this.value = changedVal;
        this.valueChange.emit(this.value);
        this.propogateValueChange();
    };
    ListValueDirective.prototype.propogateValueChange = function () {
        var _this = this;
        if (this.ngControl) {
            this.ngControl.control.patchValue(this.value);
        }
        else if (this.control) {
            this.control.controls = [];
            this.value.map(function (value) {
                return _this.control.push(new FormControl(value));
            });
        }
        this.changeRef.detectChanges();
    };
    ListValueDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: ChangeDetectorRef },
        { type: NgForm, decorators: [{ type: Optional }] },
        { type: FormGroupDirective, decorators: [{ type: Optional }] },
        { type: NgControl, decorators: [{ type: Optional }, { type: Self }] }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", NgModel)
    ], ListValueDirective.prototype, "ngModel", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], ListValueDirective.prototype, "control", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], ListValueDirective.prototype, "field", void 0);
    __decorate([
        Output(),
        __metadata("design:type", EventEmitter)
    ], ListValueDirective.prototype, "valueChange", void 0);
    __decorate([
        HostListener('anjBlur'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], ListValueDirective.prototype, "onAnjBlur", null);
    __decorate([
        HostListener('anjChange', ['$event.target']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [HTMLInputElement]),
        __metadata("design:returntype", void 0)
    ], ListValueDirective.prototype, "onAnjChange", null);
    __decorate([
        HostBinding('attr.invalid'),
        __metadata("design:type", Boolean),
        __metadata("design:paramtypes", [])
    ], ListValueDirective.prototype, "isInvalid", null);
    ListValueDirective = __decorate([
        Directive({
            selector: 'anj-list'
        }),
        __param(2, Optional()),
        __param(3, Optional()),
        __param(4, Optional()), __param(4, Self()),
        __metadata("design:paramtypes", [ElementRef,
            ChangeDetectorRef,
            NgForm,
            FormGroupDirective,
            NgControl])
    ], ListValueDirective);
    return ListValueDirective;
}());

var ZuoraDatePipe = /** @class */ (function () {
    function ZuoraDatePipe(_locale) {
        this._locale = _locale;
    }
    ZuoraDatePipe.prototype.transform = function (value, args) {
        try {
            var valueAsDate = this.convertToDate(value);
            var userOpts = args;
            var defaultOpts = { year: '2-digit', month: 'numeric', day: 'numeric' };
            var opts = Object.assign({}, defaultOpts, userOpts);
            return new Intl.DateTimeFormat(this._locale, opts).format(valueAsDate);
        }
        catch (e) {
            return value.toString();
        }
    };
    /*
        Sometimes JS is awful, handles for 'YYYY-MM-DD' to choose local timezone instead of UTC
        https://stackoverflow.com/questions/2587345/why-does-date-parse-give-incorrect-results
     */
    ZuoraDatePipe.prototype.convertToDate = function (input) {
        if (this.isIsoUtc(input)) {
            var splitString = input.split('-');
            return new Date(Number(splitString[0]), Number(splitString[1]) - 1, Number(splitString[2]));
        }
        else {
            return new Date(input);
        }
    };
    ZuoraDatePipe.prototype.isIsoUtc = function (input) {
        return typeof input === 'string' && input.charAt(4) === '-' && input.charAt(7) === '-' && input.length === 10;
    };
    ZuoraDatePipe.ctorParameters = function () { return [
        { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LOCALE_ID,] }] }
    ]; };
    ZuoraDatePipe = __decorate([
        Pipe({
            name: 'zuoraDate'
        }),
        __param(0, Optional()), __param(0, Inject(LOCALE_ID)),
        __metadata("design:paramtypes", [String])
    ], ZuoraDatePipe);
    return ZuoraDatePipe;
}());

var ToastService = /** @class */ (function () {
    function ToastService() {
        this._dismissed$ = new Subject();
    }
    ToastService_1 = ToastService;
    Object.defineProperty(ToastService.prototype, "dismissed$", {
        get: function () {
            return this._dismissed$.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    ToastService.createController = function () {
        var controller = document.createElement('anj-toast-controller');
        document.body.appendChild(controller);
        return controller;
    };
    ToastService.prototype.dismissAll = function () {
        return __awaiter(this, void 0, void 0, function () {
            var controller;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getController()];
                    case 1:
                        controller = _a.sent();
                        return [2 /*return*/, controller.dismissAll()];
                }
            });
        });
    };
    ToastService.prototype.error = function (message, header) {
        return __awaiter(this, void 0, void 0, function () {
            var controller;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getController()];
                    case 1:
                        controller = _a.sent();
                        return [2 /*return*/, controller.error(message, header)];
                }
            });
        });
    };
    ToastService.prototype.message = function (message, header) {
        return __awaiter(this, void 0, void 0, function () {
            var controller;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getController()];
                    case 1:
                        controller = _a.sent();
                        return [2 /*return*/, controller.message(message, header)];
                }
            });
        });
    };
    ToastService.prototype.success = function (message, header) {
        return __awaiter(this, void 0, void 0, function () {
            var controller;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getController()];
                    case 1:
                        controller = _a.sent();
                        return [2 /*return*/, controller.success(message, header)];
                }
            });
        });
    };
    ToastService.prototype.warn = function (message, header) {
        return __awaiter(this, void 0, void 0, function () {
            var controller;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getController()];
                    case 1:
                        controller = _a.sent();
                        return [2 /*return*/, controller.warn(message, header)];
                }
            });
        });
    };
    ToastService.prototype.getController = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.controller || this.findController()];
            });
        });
    };
    ToastService.prototype.findController = function () {
        var controller = this.findOrCreate(document.querySelector('anj-toast-controller'));
        controller.addEventListener('anjToastDidDismiss', this.onToastDidDismiss.bind(this));
        this.controller = controller;
        return controller.componentOnReady();
    };
    ToastService.prototype.onToastDidDismiss = function (event) {
        this._dismissed$.next(event);
    };
    ToastService.prototype.findOrCreate = function (control) {
        return control || ToastService_1.createController();
    };
    var ToastService_1;
    ToastService.ɵprov = ɵɵdefineInjectable({ factory: function ToastService_Factory() { return new ToastService(); }, token: ToastService, providedIn: "root" });
    ToastService = ToastService_1 = __decorate([
        Injectable({ providedIn: 'root' })
    ], ToastService);
    return ToastService;
}());

var LocalStorageService = /** @class */ (function (_super) {
    __extends(LocalStorageService, _super);
    function LocalStorageService() {
        return _super.call(this) || this;
    }
    LocalStorageService.ɵprov = ɵɵdefineInjectable({ factory: function LocalStorageService_Factory() { return new LocalStorageService(); }, token: LocalStorageService, providedIn: "root" });
    LocalStorageService = __decorate([
        Injectable({ providedIn: 'root' }),
        __metadata("design:paramtypes", [])
    ], LocalStorageService);
    return LocalStorageService;
}(LocalStorage));
var SessionStorageService = /** @class */ (function (_super) {
    __extends(SessionStorageService, _super);
    function SessionStorageService() {
        return _super.call(this) || this;
    }
    SessionStorageService.ɵprov = ɵɵdefineInjectable({ factory: function SessionStorageService_Factory() { return new SessionStorageService(); }, token: SessionStorageService, providedIn: "root" });
    SessionStorageService = __decorate([
        Injectable({ providedIn: 'root' }),
        __metadata("design:paramtypes", [])
    ], SessionStorageService);
    return SessionStorageService;
}(SessionStorage));

var DECLARATIONS = [
    // Common
    DraggableDirective,
    DroppableDirective,
    LocaleDirective,
    TemplateDirective,
    TransitionDirective,
    ZuoraDatePipe,
    // Forms
    BooleanValueDirective,
    DatePickerDirective,
    DatetimePickerDirective,
    DateValidatorDirective,
    ListValueDirective,
    RequiredCheckDirective,
    SelectValueDirective,
    TextValueDirective,
    TimePickerDirective,
    // Tables
    AnjTable
];
var AnjunaCoreModule = /** @class */ (function () {
    function AnjunaCoreModule() {
    }
    AnjunaCoreModule = __decorate([
        NgModule({
            declarations: DECLARATIONS,
            exports: DECLARATIONS,
            providers: [
                {
                    provide: TIME_MERIDIEM,
                    useValue: true
                },
                ZuoraDatePipe
            ]
        })
    ], AnjunaCoreModule);
    return AnjunaCoreModule;
}());

/* Abstracts */

/**
 * Generated bundle index. Do not edit.
 */

export { AnjTable, AnjunaCoreModule, AnjunaValidators, BaseTranslationPipe, BooleanValueDirective, DATE_TIMEZONE_OVERRIDE, DatePickerDirective, DateValidatorDirective, DatetimePickerDirective, DomOutlet, DraggableDirective, DroppableDirective, LANGUAGE_CODE, ListValueDirective, LocalStorageService, LocaleDirective, RequiredCheckDirective, SelectValueDirective, SessionStorageService, TIME_MERIDIEM, TIME_TIMEZONE_STRING, Template, TemplateDirective, TextValueDirective, TimePickerDirective, ToastService, TransitionDirective, TransitionState, ZuoraDatePipe, ValueAccessor as ɵa, PickerAbstract as ɵb };
//# sourceMappingURL=anjuna-angular-core.js.map
